if(1) // This line has no spaces
if (1) // This line has one space
  {
  if  (1) // This line has two spaces
    {
    if( 0 ) // This line has one space
      {
      int iffy = 0;  //Verify string matching eliminates false tests
      int iif = 0;
      int iif=0;
      int ifif = 0;
      int ifif=0;
      if
        ( iif == 0 )  // should always be an error
      }
    }
  }
