#ifndef __9934-test_h
#define __9934-test_h

/** \class Test */
class MyAwesomeBug : public QObject
{
public:

  void switchToMyAwesomeBug(const QString& myAwesomeBugName)
    {
    return;
    }
};

#endif
