/*=========================================================================

  Program:   CMake - Cross-Platform Makefile Generator
  Module:    CMakeHeader.h
  Language:  C++
  Date:      $Date$
  Version:   $Revision$

  Copyright (c) 2002-2020 Kitware, Inc., NumFOCUS  All rights reserved.
  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
