if (1) // this line has no tabs
  { // this line has no tabs
	// this line has a leading tab
  // this line has a tab 	in the middle
  // this line has a tab at the end	
	// this			line has lots of			tabs			
  } // this line has no tabs
